/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.plugin.view;

import org.opengion.hayabusa.common.HybsSystem;
import org.opengion.fukurou.util.StringUtil;

/**
 * 行と列を入れ替えて表示する、テーブル回転表示クラスです。
 *
 * このクラスは、表示のみ実行可能です。旧ヘッダー部分は、第一カラムに表示されます。
 * このビューでは、行と列が入れ替わって表示している為、登録はできません。
 *
 * AbstractViewForm により、setter/getterメソッドのデフォルト実装を提供しています。
 * 各HTMLのタグに必要な setter/getterメソッドのみ，追加定義しています。
 *
 * AbstractViewForm を継承している為,ロケールに応じたラベルを出力させる事が出来ます。
 *
 * @og.rev 3.5.3.0 (2003/10/27) 新規作成
 * @og.group 画面表示
 *
 * @version  4.0
 * @author   Kazuhiko Hasegawa
 * @since    JDK5.0,
 */
public class ViewForm_HTMLRotationTable extends ViewForm_HTMLTable  {
	//* このプログラムのVERSION文字列を設定します。	{@value} */
	private static final String VERSION = "4.0.0 (2005/08/31)" ;

	private boolean	 noClass = false;

	// 4.3.4.4 (2009/01/01)
//	/**
//	 * デフォルトコンストラクター
//	 *
//	 */
//	public ViewForm_HTMLRotationTable() {
//		super();
//		super.clear();
//	}

	/**
	 * DBTableModel から HTML文字列を作成して返します。
	 * startNo（表示開始位置）から、pageSize（表示件数）までのView文字列を作成します。
	 * 表示残りデータが pageSize 以下の場合は,残りのデータをすべて出力します。
	 *
	 * @og.rev 3.5.3.1 (2003/10/31) BgColorCycleClass の設定不具合修正。
	 * @og.rev 3.5.6.4 (2004/07/16) ヘッダーとボディー部をJavaScriptで分離
	 * @og.rev 3.8.8.5 (2007/03/09) 表示の仕方を修正しました。
	 * @og.rev 4.0.0 (2005/01/31) 新規作成（getColumnClassName ⇒ getColumnDbType）
	 *
	 * @param  startNo    表示開始位置
	 * @param  pageSize   表示件数
	 * @return  DBTableModel から作成された HTML文字列
	 */
	public String create( final int startNo, final int pageSize )  {
		if( getRowCount() == 0 ) { return ""; }	// 暫定処置

		int lastNo = getLastNo( startNo, pageSize );

		StringBuilder out = new StringBuilder( HybsSystem.BUFFER_LARGE );

		out.append( getCountForm( startNo,pageSize ) );
		out.append( getHeader() );

		String clmCls = "<td>";
		out.append("<tbody>").append( HybsSystem.CR );
		int bgClrCnt = 0;
		int clmCnt = getColumnCount();	// 3.5.5.7 (2004/05/10)
		for(int column = 0; column < clmCnt; column++) {
			if( !isColumnDisplay( column ) ) { continue; }
			int dummyRow = bgClrCnt++ ;		// 3.8.8.5 (2007/03/09)

			out.append("<tr").append( getBgColorCycleClass( dummyRow ) ).append(">");
			out.append( HybsSystem.CR );
			// 3.8.8.5 (2007/03/09) numberType 属性を考慮
			if( isNumberDisplay() ) {
				out.append( "<td>" ).append( getNumberData( dummyRow ) ).append( "</td>" );
				out.append( HybsSystem.CR );
			}
			out.append( "<td>" );
			out.append( getColumnLabel(column) ).append("</td>");
			out.append( HybsSystem.CR );

			if( ! noClass ) {
				clmCls = "<td class=\"" + getColumnDbType(column) + "\" >" ;	// 4.0.0 (2005/01/31)
			}

			for( int row=startNo; row<lastNo; row++ ) {
				out.append( clmCls );
				out.append( getValueLabel(row,column) );
				out.append("</td>").append( HybsSystem.CR );
			}
			out.append("</tr>").append( HybsSystem.CR );
		}
		out.append("</tbody>").append( HybsSystem.CR );
		out.append("</table>").append( HybsSystem.CR );

		out.append( getScrollBarEndDiv() );	// 3.8.0.3 (2005/07/15)
		return out.toString();
	}

	/**
	 * 内容をクリア（初期化）します。
	 *
	 */
	public void clear() {
		super.clear();
		noClass 		= false;
	}

	/**
	 * テーブルのバックグラウンドカラーの値をセットします。<br />
	 * 行番号は, ０から始まるので、偶数を HTML_BG_COLOR_ROW0 、
	 * 奇数行を HTML_BG_COLOR_ROW1 とします。<br />
	 * setBgColorCycle で、設定値変換しています。
	 * なお、このクラスでは、最初の行に、row_h クラス属性を付加します。
	 *
	 * @og.rev 3.8.8.5 (2007/03/09) ヘッダー部の色付け
	 *
	 * @param	indx 先頭からの連番（ ０から始める ）
	 * @return	行の色を指定する class 属性（ cssファイルで指定 ）
	 */
	protected String getBgColorCycleClass( final int indx ) {
		return ( indx == 0 )
					? " class=\"row_h\""
					: super.getBgColorCycleClass( indx );
	}

	/**
	 * DBTableModel から テーブルのタグ文字列を作成して返します。
	 *
	 * @return  テーブルのタグ文字列
	 */
	protected String getTableHead() {

		StringBuilder buf = new StringBuilder( HybsSystem.BUFFER_MIDDLE );
		if( isNumberDisplay() ) {
			buf.append("<colgroup class=\"S9\">");
		}
		buf.append("<colgroup class=\"HEADER\" />");
		buf.append(HybsSystem.CR);

		return buf.toString();
	}

	/**
	 * フォーマットメソッドを使用できるかどうかを問い合わせます。
	 *
	 * @return  使用可能（true)/ 使用不可能 (false)
	 */
	public boolean canUseFormat() {
		return false;
	}

	/**
	 * カラムのクラス名（<de>VERCHAR2 , NUMBER</del>X,S9 など）のセットを行うかどうか指定します。
	 *
	 * "true" で、クラス属性を設定しません。これは、ＣＳＳファイルに書かれている属性を
	 * 使用しないことを意味します。
	 * デフォルトは、"false" です。
	 *
	 * @param	flag クラス名使用の有無（true:使用しない/false:使用する。）
	 */
	public void setBodyNoClass( final String flag ) {
		noClass = StringUtil.nval( flag,noClass );
	}
}
